# -*- coding: utf8 -*-
from __future__ import unicode_literals


from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    Segment,
    HprimList,
    HprimMultiple,
    HprimString,
    HprimDate,
    HprimForeign,
)


class IndikoSegmentH(HeaderSegment):
    analyzer_type = HprimString((5, 1))
    analyzer_id = HprimString((5, 2))
    version = HprimString((5, 3))
    processing_id = HprimString(12, default="P")
    date = HprimDate(14)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class IndikoSegmentP(Segment):
    patient_id = HprimString(3)
    patient_name = HprimString(6)
    date_birth = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    ordering_doctor = HprimString(26)

    orders = HprimForeign("O", "patient")


class IndikoSegmentO(Segment):
    sample_id = HprimString((3, 1))
    rack = HprimString((3, 2))
    position = HprimString((3, 3))
    tests = HprimList(5, basefield=HprimMultiple(
        0,
        test_code=HprimString(4),
        dilution=HprimString(5)
    ))
    # S: Stat
    # A: Asap
    # R: Routine
    priority = HprimString(6)
    collection_date = HprimDate(8)
    # A: Add
    # N: New
    # P: Pending sample
    # X: Sample or test in process
    # Q: QC sample
    # C: Comment
    action_code = HprimString(12)
    clinical_info = HprimString(14)
    sample_descriptor = HprimString(16)
    laboratory_field_no1 = HprimString(21)
    instrument = HprimString(25)
    report_type = HprimString(26)
    results = HprimForeign("R", "order")


class IndikoSegmentR(Segment):
    test_id = HprimString((3, 4))
    value = HprimString(4)
    units = HprimString(5)
    reference_ranges = HprimString(6)
    abnormal_flags = HprimString(8)
    result_status = HprimString(9)
    operator = HprimString(11)
    date_completed = HprimDate(13)
    analyzer = HprimString(14)


class IndikoSegmentC(Segment):
    comment_source = HprimString(3)
    comment_text = HprimList(4)
    comment_type = HprimString(5)


class IndikoSegmentQ(Segment):
    sample_id = HprimString((3, 2))
    rack = HprimString((3, 3))
    position = HprimString((3, 4))
    test_id = HprimString((5, 4))
    nature_requests = HprimString((6, 2))
    request_info = HprimString(13)


class IndikoSegmentL(Segment):
    terminaison_code = HprimString(3, default="N")


class Driver(AstmMessageV2):
    SPEC = {
        "code": "indiko",
        "label": "Indiko Gallery ASTM"
    }

    SEGMENT_CLASS_MAP = {
        "H": IndikoSegmentH,
        "P": IndikoSegmentP,
        "O": IndikoSegmentO,
        "R": IndikoSegmentR,
        "C": IndikoSegmentC,
        "Q": IndikoSegmentQ,
        "L": IndikoSegmentL,
    }

    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "indiko_qc_prefix", label="Préfixe QC"),
        AstmMessageV2.Option(
            "indiko_envoyer_no_patient_id",
            label="Envoyer le numéro IP Patient de MCA dans le champ P.3",
        )
    ]

    def __init__(self, indiko_envoyer_no_patient_id=False, indiko_qc_prefix='', **kwargs):
        self.envoyer_no_patient_id = indiko_envoyer_no_patient_id
        self.qc_prefix = indiko_qc_prefix
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = IndikoSegmentH(type_segment="H", **kwargs)
        message.end = IndikoSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"A": "A", "S": "S", worklist.ELSE: "R"},
            action={"del": worklist.IGNORE, "add": "A", "new": "N"}
        ):
            patient_name = "{} {}".format(
                patient.last_name,
                patient.first_name
            ).strip()
            seg_p = msg.patients.add(
                patient_id=patient.patient_id if self.envoyer_no_patient_id else patient.lab_patient_id,
                patient_name=patient_name,
                date_birth=patient.birthdate,
                sex=patient.sex,
            )
            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    priority=priority,
                    action_code=action,
                    collection_date=sample.collection_date,
                    sample_descriptor=sample.specimen,
                    report_type="Q" if context["query"] else "O"
                )
                for order in sample.orders:
                    t = seg_o.tests.add()
                    t.test_code = order["code"]
                    t.dilution = order["dilution"]
                for physician in sample.physicians:
                    seg_p.ordering_doctor = physician.code
        if msg.patients:
            return msg
        elif context["query"] and context["query"].queries:
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                if seg_q.rack and seg_q.position:
                    rack = "{}-{}".format(
                        seg_q.rack,
                        seg_q.position
                    )
                else:
                    rack = ""
                worklist.get_or_create(
                    type="patient",
                    sample_id=seg_q.sample_id,
                    rack=rack
                )
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    if seg_o.action_code == "Q" or (self.qc_prefix and seg_o.sample_id.startswith(self.qc_prefix)):
                        sample = worklist.get_or_create(
                            sample_id=seg_o.laboratory_field_no1 if seg_o.laboratory_field_no1 else seg_o.sample_id,  # ref ou n° lot
                            control_lot=seg_o.laboratory_field_no1 if seg_o.laboratory_field_no1 else seg_o.sample_id,
                            type="control"
                        )
                    else:
                        sample = worklist.get_or_create(
                            sample_id=seg_o.sample_id,
                            type="patient"
                        )
                        if seg_o.rack and seg_o.position:
                            sample.rack = "{}-{}".format(
                                seg_o.rack,
                                seg_o.position
                            )
                    comments = []
                    for seg_r in seg_o.results:
                        flags = []
                        if seg_r.abnormal_flags:
                            flags.append(seg_r.abnormal_flags)
                        for seg_c in seg_r.commentaires:
                            if seg_c.comment_type == 'I':
                                for f in seg_c.comment_text:
                                    flags.append(f)
                            else:
                                comments.extend(seg_c.comment_text)
                        sample.results.add(
                            code=seg_r.test_id,
                            value=seg_r.value,
                            unit=seg_r.units,
                            flags=flags,
                            comments=comments,
                            date=seg_r.date_completed,
                            operator=seg_r.operator,
                            instrument=seg_r.analyzer
                        )

        return worklist
